/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.netbeans.modules.nbcode.integration;

import java.util.concurrent.atomic.AtomicBoolean;
import org.netbeans.modules.progress.spi.RunOffEDTProvider;
import org.openide.util.Mutex;
import org.openide.util.RequestProcessor;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author sdedic
 */
@ServiceProvider(service = RunOffEDTProvider.class, position = 90)
public class LspRunOffEdtProvider implements RunOffEDTProvider {
    
    private static final RequestProcessor TRIVIAL = new RequestProcessor(LspRunOffEdtProvider.class);
    
    @Override
    public void runOffEventDispatchThread(Runnable operation, String operationDescr, AtomicBoolean cancelOperation, boolean waitForCanceled, int waitCursorAfter, int dialogAfter) {
        if (Mutex.EVENT.isReadAccess()) {
            // PENDING: the EDT should continue to service events.
            TRIVIAL.post(operation).waitFinished();
        } else {
            operation.run();
        }
    }
}
